@@ -17,7 +17,7 @@
 
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: client.c,v 1.84 2005/06/12 16:39:42 alex Exp $";
+static char UNUSED id[] = "$Id: client.c,v 1.91 2006/04/23 10:37:27 fw Exp $";
 
 #include "imp.h"
 #include <assert.h>
@@ -50,19 +50,23 @@ static char UNUSED id[] = "$Id: client.c,v 1.84 2005/06/12 16:39:42 alex Exp $";
 #define GETID_LEN (CLIENT_NICK_LEN-1) + 1 + (CLIENT_USER_LEN-1) + 1 + (CLIENT_HOST_LEN-1) + 1
 
 
-LOCAL CLIENT *This_Server, *My_Clients;
-LOCAL char GetID_Buffer[GETID_LEN];
+static CLIENT *This_Server, *My_Clients;
+static char GetID_Buffer[GETID_LEN];
 
-LOCAL WHOWAS My_Whowas[MAX_WHOWAS];
-LOCAL int Last_Whowas = -1;
+static WHOWAS My_Whowas[MAX_WHOWAS];
+static int Last_Whowas = -1;
 
 
-LOCAL long Count PARAMS(( CLIENT_TYPE Type ));
-LOCAL long MyCount PARAMS(( CLIENT_TYPE Type ));
+static long Count PARAMS(( CLIENT_TYPE Type ));
+static long MyCount PARAMS(( CLIENT_TYPE Type ));
 
-LOCAL CLIENT *New_Client_Struct PARAMS(( void ));
-LOCAL void Generate_MyToken PARAMS(( CLIENT *Client ));
-LOCAL void Adjust_Counters PARAMS(( CLIENT *Client ));
+static CLIENT *New_Client_Struct PARAMS(( void ));
+static void Generate_MyToken PARAMS(( CLIENT *Client ));
+static void Adjust_Counters PARAMS(( CLIENT *Client ));
+
+static CLIENT *Init_New_Client PARAMS((CONN_ID Idx, CLIENT *Introducer,
+ CLIENT *TopServer, int Type, char *ID, char *User, char *Hostname,
+ char *Info, int Hops, int Token, char *Modes, bool Idented));
 
 #ifndef Client_DestroyNow
 GLOBAL void Client_DestroyNow PARAMS((CLIENT *Client ));
@@ -135,32 +139,53 @@ Client_ThisServer( void )
 } /* Client_ThisServer */
 
 
+/**
+ * Initialize new local client; wrapper function for Init_New_Client().
+ * @return New CLIENT structure.
+ */
 GLOBAL CLIENT *
-Client_NewLocal( CONN_ID Idx, char *Hostname, int Type, bool Idented )
+Client_NewLocal(CONN_ID Idx, char *Hostname, int Type, bool Idented)
 {
-	/* Neuen lokalen Client erzeugen: Wrapper-Funktion fuer Client_New(). */
-	return Client_New( Idx, This_Server, NULL, Type, NULL, NULL, Hostname, NULL, 0, 0, NULL, Idented );
+	return Init_New_Client(Idx, This_Server, NULL, Type, NULL, NULL,
+		Hostname, NULL, 0, 0, NULL, Idented);
 } /* Client_NewLocal */
 
 
+/**
+ * Initialize new remote server; wrapper function for Init_New_Client().
+ * @return New CLIENT structure.
+ */
 GLOBAL CLIENT *
-Client_NewRemoteServer( CLIENT *Introducer, char *Hostname, CLIENT *TopServer, int Hops, int Token, char *Info, bool Idented )
+Client_NewRemoteServer(CLIENT *Introducer, char *Hostname, CLIENT *TopServer,
+ int Hops, int Token, char *Info, bool Idented)
 {
-	/* Neuen Remote-Client erzeugen: Wrapper-Funktion fuer Client_New (). */
-	return Client_New( NONE, Introducer, TopServer, CLIENT_SERVER, Hostname, NULL, Hostname, Info, Hops, Token, NULL, Idented );
+	return Init_New_Client(NONE, Introducer, TopServer, CLIENT_SERVER,
+		Hostname, NULL, Hostname, Info, Hops, Token, NULL, Idented);
 } /* Client_NewRemoteServer */
 
 
+/**
+ * Initialize new remote client; wrapper function for Init_New_Client().
+ * @return New CLIENT structure.
+ */
 GLOBAL CLIENT *
-Client_NewRemoteUser( CLIENT *Introducer, char *Nick, int Hops, char *User, char *Hostname, int Token, char *Modes, char *Info, bool Idented )
+Client_NewRemoteUser(CLIENT *Introducer, char *Nick, int Hops, char *User,
+ char *Hostname, int Token, char *Modes, char *Info, bool Idented)
 {
-	/* Neuen Remote-Client erzeugen: Wrapper-Funktion fuer Client_New (). */
-	return Client_New( NONE, Introducer, NULL, CLIENT_USER, Nick, User, Hostname, Info, Hops, Token, Modes, Idented );
+	return Init_New_Client(NONE, Introducer, NULL, CLIENT_USER, Nick,
+		User, Hostname, Info, Hops, Token, Modes, Idented);
 } /* Client_NewRemoteUser */
 
 
-GLOBAL CLIENT *
-Client_New( CONN_ID Idx, CLIENT *Introducer, CLIENT *TopServer, int Type, char *ID, char *User, char *Hostname, char *Info, int Hops, int Token, char *Modes, bool Idented )
+/**
+ * Initialize new client and set up the given parameters like client type,
+ * user name, host name, introducing server etc. ...
+ * @return New CLIENT structure.
+ */
+static CLIENT *
+Init_New_Client(CONN_ID Idx, CLIENT *Introducer, CLIENT *TopServer,
+ int Type, char *ID, char *User, char *Hostname, char *Info, int Hops,
+ int Token, char *Modes, bool Idented)
 {
 	CLIENT *client;
 
@@ -197,7 +222,7 @@ Client_New( CONN_ID Idx, CLIENT *Introducer, CLIENT *TopServer, int Type, char *
 	Adjust_Counters( client );
 
 	return client;
-} /* Client_New */
+} /* Init_New_Client */
 
 
 GLOBAL void
@@ -296,11 +321,9 @@ Client_Destroy( CLIENT *Client, char *LogMsg, char *FwdMsg, bool SendQuit )
 				{
 					if( c->id[0] ) Log( LOG_NOTICE, "Client \"%s\" unregistered (connection %d): %s", c->id, c->conn_id, txt );
 					else Log( LOG_NOTICE, "Client unregistered (connection %d): %s", c->conn_id, txt );
-				}
-				else
-				{
-					if( c->id[0] ) Log( LOG_WARNING, "Unregistered unknown client \"%s\": %s", c->id, txt );
-					else Log( LOG_WARNING, "Unregistered unknown client: %s", c->id, txt );
+				} else {
+					Log(LOG_WARNING, "Unregistered unknown client \"%s\": %s",
+								c->id[0] ? c->id : "(No Nick)", txt );
 				}
 			}
 
@@ -540,27 +563,6 @@ Client_ModeDel( CLIENT *Client, char Mode )
 } /* Client_ModeDel */
 
 
-GLOBAL CLIENT *
-Client_GetFromConn( CONN_ID Idx )
-{
-	/* return Client-Structure that belongs to the local Connection Idx gehoert.
-	 * If none is found, return NULL.
-	 */
-
-	CLIENT *c;
-
-	assert( Idx >= 0 );
-	
-	c = My_Clients;
-	while( c )
-	{
-		if( c->conn_id == Idx ) return c;
-		c = (CLIENT *)c->next;
-	}
-	return NULL;
-} /* Client_GetFromConn */
-
-
 GLOBAL CLIENT *
 Client_Search( char *Nick )
 {
@@ -1035,7 +1037,7 @@ Client_StartTime(CLIENT *Client)
 } /* Client_Uptime */
 
 
-LOCAL long
+static long
 Count( CLIENT_TYPE Type )
 {
 	CLIENT *c;
@@ -1052,7 +1054,7 @@ Count( CLIENT_TYPE Type )
 } /* Count */
 
 
-LOCAL long
+static long
 MyCount( CLIENT_TYPE Type )
 {
 	CLIENT *c;
@@ -1069,7 +1071,7 @@ MyCount( CLIENT_TYPE Type )
 } /* MyCount */
 
 
-LOCAL CLIENT *
+static CLIENT *
 New_Client_Struct( void )
 {
 	/* Neue CLIENT-Struktur pre-initialisieren */
@@ -1096,7 +1098,7 @@ New_Client_Struct( void )
 } /* New_Client */
 
 
-LOCAL void
+static void
 Generate_MyToken( CLIENT *Client )
 {
 	CLIENT *c;
@@ -1120,7 +1122,7 @@ Generate_MyToken( CLIENT *Client )
 } /* Generate_MyToken */
 
 
-LOCAL void
+static void
 Adjust_Counters( CLIENT *Client )
 {
 	long count;
@@ -1150,11 +1152,13 @@ GLOBAL void
 Client_RegisterWhowas( CLIENT *Client )
 {
 	int slot;
+	time_t now;
 	
 	assert( Client != NULL );
 
+	now = time(NULL);
 	/* Don't register clients that were connected less than 30 seconds. */
-	if( time(NULL) - Client->starttime < 30 )
+	if( now - Client->starttime < 30 )
 		return;
 
 	slot = Last_Whowas + 1;
@@ -1164,7 +1168,7 @@ Client_RegisterWhowas( CLIENT *Client )
 	Log( LOG_DEBUG, "Saving WHOWAS information to slot %d ...", slot );
 #endif
 	
-	My_Whowas[slot].time = time( NULL );
+	My_Whowas[slot].time = now;
 	strlcpy( My_Whowas[slot].id, Client_ID( Client ),
 		 sizeof( My_Whowas[slot].id ));
 	strlcpy( My_Whowas[slot].user, Client_User( Client ),
