@@ -14,7 +14,7 @@
 
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: conf.c,v 1.77.2.1 2005/10/11 19:28:47 alex Exp $";
+static char UNUSED id[] = "$Id: conf.c,v 1.92 2006/07/23 16:42:45 alex Exp $";
 
 #include "imp.h"
 #include <assert.h>
@@ -38,6 +38,7 @@ static char UNUSED id[] = "$Id: conf.c,v 1.77.2.1 2005/10/11 19:28:47 alex Exp $
 # include <ctype.h>
 #endif
 
+#include "array.h"
 #include "ngircd.h"
 #include "conn.h"
 #include "client.h"
@@ -50,26 +51,84 @@ static char UNUSED id[] = "$Id: conf.c,v 1.77.2.1 2005/10/11 19:28:47 alex Exp $
 #include "conf.h"
 
 
-LOCAL bool Use_Log = true;
-LOCAL CONF_SERVER New_Server;
-LOCAL int New_Server_Idx;
+static bool Use_Log = true;
+static CONF_SERVER New_Server;
+static int New_Server_Idx;
 
 
-LOCAL void Set_Defaults PARAMS(( bool InitServers ));
-LOCAL void Read_Config PARAMS(( void ));
-LOCAL void Validate_Config PARAMS(( bool TestOnly ));
+static void Set_Defaults PARAMS(( bool InitServers ));
+static void Read_Config PARAMS(( void ));
+static void Validate_Config PARAMS(( bool TestOnly, bool Rehash ));
 
-LOCAL void Handle_GLOBAL PARAMS(( int Line, char *Var, char *Arg ));
-LOCAL void Handle_OPERATOR PARAMS(( int Line, char *Var, char *Arg ));
-LOCAL void Handle_SERVER PARAMS(( int Line, char *Var, char *Arg ));
-LOCAL void Handle_CHANNEL PARAMS(( int Line, char *Var, char *Arg ));
+static void Handle_GLOBAL PARAMS(( int Line, char *Var, char *Arg ));
+static void Handle_OPERATOR PARAMS(( int Line, char *Var, char *Arg ));
+static void Handle_SERVER PARAMS(( int Line, char *Var, char *Arg ));
+static void Handle_CHANNEL PARAMS(( int Line, char *Var, char *Arg ));
 
-LOCAL void Config_Error PARAMS(( const int Level, const char *Format, ... ));
+static void Config_Error PARAMS(( const int Level, const char *Format, ... ));
 
-LOCAL void Config_Error_NaN PARAMS(( const int LINE, const char *Value ));
-LOCAL void Config_Error_TooLong PARAMS(( const int LINE, const char *Value ));
+static void Config_Error_NaN PARAMS(( const int LINE, const char *Value ));
+static void Config_Error_TooLong PARAMS(( const int LINE, const char *Value ));
 
-LOCAL void Init_Server_Struct PARAMS(( CONF_SERVER *Server ));
+static void Init_Server_Struct PARAMS(( CONF_SERVER *Server ));
+
+
+static char *
+strdup_warn(const char *str)
+{
+	char *ptr = strdup(str);
+	if (!ptr)
+		Config_Error(LOG_ERR, "Could not allocate mem for string: %s", str);
+	return ptr;
+}
+
+
+static void
+ports_puts(array *a)
+{
+	size_t len;
+	UINT16 *ports;
+	len = array_length(a, sizeof(UINT16));
+	if (len--) {
+		ports = (UINT16*) array_start(a);
+		printf("%u", (unsigned int) *ports);
+		while (len--) {
+			ports++;
+			printf(", %u", (unsigned int) *ports);
+		}
+	}
+	putc('\n', stdout);
+}
+
+
+static void
+ports_parse(array *a, int Line, char *Arg)
+{
+	char *ptr;
+	int port;
+	UINT16 port16;
+
+	array_trunc(a);
+
+	/* Ports on that the server should listen. More port numbers
+	 * must be separated by "," */
+	ptr = strtok( Arg, "," );
+	while (ptr) {
+		ngt_TrimStr( ptr );
+		port = atol( ptr );
+		if (port > 0 && port < 0xFFFF) {
+			port16 = (UINT16) port;
+			if (!array_catb(a, (char*)&port16, sizeof port16))
+				Config_Error(LOG_ERR, "%s, line %d Could not add port number %ld: %s",
+							NGIRCd_ConfFile, Line, port, strerror(errno));
+		} else {
+			Config_Error( LOG_ERR, "%s, line %d (section \"Global\"): Illegal port number %ld!",
+									NGIRCd_ConfFile, Line, port );
+		}
+
+		ptr = strtok( NULL, "," );
+	}
+}
 
 
 GLOBAL void
@@ -77,7 +136,7 @@ Conf_Init( void )
 {
 	Set_Defaults( true );
 	Read_Config( );
-	Validate_Config( false );
+	Validate_Config(false, false);
 } /* Config_Init */
 
 
@@ -86,7 +145,7 @@ Conf_Rehash( void )
 {
 	Set_Defaults( false );
 	Read_Config( );
-	Validate_Config( false );
+	Validate_Config(false, true);
 } /* Config_Rehash */
 
 
@@ -97,22 +156,23 @@ Conf_Test( void )
 
 	struct passwd *pwd;
 	struct group *grp;
-	int i;
+	unsigned int i;
+	char *topic;
 
 	Use_Log = false;
 	Set_Defaults( true );
 
 	Read_Config( );
-	Validate_Config( true );
+	Validate_Config(true, false);
 
 	/* If stdin and stdout ("you can read our nice message and we can
 	 * read in your keypress") are valid tty's, wait for a key: */
-	if( isatty( fileno( stdin )) && isatty( fileno( stdout )))
-	{
+	if( isatty( fileno( stdin )) && isatty( fileno( stdout ))) {
 		puts( "OK, press enter to see a dump of your service configuration ..." );
 		getchar( );
+	} else {
+		puts( "Ok, dump of your server configuration follows:\n" );
 	}
-	else puts( "Ok, dump of your server configuration follows:\n" );
 
 	puts( "[GLOBAL]" );
 	printf( "  Name = %s\n", Conf_ServerName );
@@ -125,13 +185,10 @@ Conf_Test( void )
 	printf( "  MotdPhrase = %s\n", Conf_MotdPhrase );
 	printf( "  ChrootDir = %s\n", Conf_Chroot );
 	printf( "  PidFile = %s\n", Conf_PidFile);
-	printf( "  Ports = " );
-	for( i = 0; i < Conf_ListenPorts_Count; i++ )
-	{
-		if( i != 0 ) printf( ", " );
-		printf( "%u", (unsigned int) Conf_ListenPorts[i] );
-	}
-	puts( "" );
+	fputs("  Ports = ", stdout);
+
+	ports_puts(&Conf_ListenPorts);
+
 	printf( "  Listen = %s\n", Conf_ListenAddress );
 	pwd = getpwuid( Conf_UID );
 	if( pwd ) printf( "  ServerUID = %s\n", pwd->pw_name );
@@ -144,16 +201,11 @@ Conf_Test( void )
 	printf( "  ConnectRetry = %d\n", Conf_ConnectRetry );
 	printf( "  OperCanUseMode = %s\n", Conf_OperCanMode == true? "yes" : "no" );
 	printf( "  OperServerMode = %s\n", Conf_OperServerMode == true? "yes" : "no" );
-	if( Conf_MaxConnections > 0 ) printf( "  MaxConnections = %ld\n", Conf_MaxConnections );
-	else printf( "  MaxConnections = -1\n" );
-	if( Conf_MaxConnectionsIP > 0 ) printf( "  MaxConnectionsIP = %d\n", Conf_MaxConnectionsIP );
-	else printf( "  MaxConnectionsIP = -1\n" );
-	if( Conf_MaxJoins > 0 ) printf( "  MaxJoins = %d\n", Conf_MaxJoins );
-	else printf( "  MaxJoins = -1\n" );
-	puts( "" );
+	printf( "  MaxConnections = %ld\n", Conf_MaxConnections>0 ? Conf_MaxConnections : -1);
+	printf( "  MaxConnectionsIP = %d\n", Conf_MaxConnectionsIP>0 ? Conf_MaxConnectionsIP : -1);
+	printf( "  MaxJoins = %d\n\n", Conf_MaxJoins>0 ? Conf_MaxJoins : -1);
 
-	for( i = 0; i < Conf_Oper_Count; i++ )
-	{
+	for( i = 0; i < Conf_Oper_Count; i++ ) {
 		if( ! Conf_Oper[i].name[0] ) continue;
 		
 		/* Valid "Operator" section */
@@ -164,31 +216,29 @@ Conf_Test( void )
 		puts( "" );
 	}
 
-	for( i = 0; i < MAX_SERVERS; i++ )
-	{
+	for( i = 0; i < MAX_SERVERS; i++ ) {
 		if( ! Conf_Server[i].name[0] ) continue;
 		
 		/* Valid "Server" section */
 		puts( "[SERVER]" );
 		printf( "  Name = %s\n", Conf_Server[i].name );
 		printf( "  Host = %s\n", Conf_Server[i].host );
-		printf( "  Port = %d\n", Conf_Server[i].port );
+		printf( "  Port = %u\n", (unsigned int)Conf_Server[i].port );
 		printf( "  MyPassword = %s\n", Conf_Server[i].pwd_in );
 		printf( "  PeerPassword = %s\n", Conf_Server[i].pwd_out );
-		printf( "  Group = %d\n", Conf_Server[i].group );
-		puts( "" );
+		printf( "  Group = %d\n\n", Conf_Server[i].group );
 	}
 
-	for( i = 0; i < Conf_Channel_Count; i++ )
-	{
+	for( i = 0; i < Conf_Channel_Count; i++ ) {
 		if( ! Conf_Channel[i].name[0] ) continue;
 		
 		/* Valid "Channel" section */
 		puts( "[CHANNEL]" );
 		printf( "  Name = %s\n", Conf_Channel[i].name );
 		printf( "  Modes = %s\n", Conf_Channel[i].modes );
-		printf( "  Topic = %s\n", Conf_Channel[i].topic );
-		puts( "" );
+
+		topic = (char*)array_start(&Conf_Channel[i].topic);
+		printf( "  Topic = %s\n\n", topic ? topic : "");
 	}
 	
 	return 0;
@@ -207,29 +257,25 @@ Conf_UnsetServer( CONN_ID Idx )
 	time_t t;
 
 	/* Check all our configured servers */
-	for( i = 0; i < MAX_SERVERS; i++ )
-	{
+	for( i = 0; i < MAX_SERVERS; i++ ) {
 		if( Conf_Server[i].conn_id != Idx ) continue;
 
 		/* Gotcha! Mark server configuration as "unused": */
 		Conf_Server[i].conn_id = NONE;
 
-		if( Conf_Server[i].flags & CONF_SFLAG_ONCE )
-		{
+		if( Conf_Server[i].flags & CONF_SFLAG_ONCE ) {
 			/* Delete configuration here */
 			Init_Server_Struct( &Conf_Server[i] );
-		}
-		else
-		{
+		} else {
 			/* Set time for next connect attempt */
-			t = time(NULL);
-			if (Conf_Server[i].lasttry < t - Conf_ConnectRetry) {
-				/* The connection has been "long", so we don't
-				 * require the next attempt to be delayed. */
-				Conf_Server[i].lasttry =
-					t - Conf_ConnectRetry + RECONNECT_DELAY;
-			} else
-				Conf_Server[i].lasttry = t;
+ 			t = time(NULL);
+ 			if (Conf_Server[i].lasttry < t - Conf_ConnectRetry) {
+ 				/* The connection has been "long", so we don't
+ 				 * require the next attempt to be delayed. */
+ 				Conf_Server[i].lasttry =
+ 					t - Conf_ConnectRetry + RECONNECT_DELAY;
+ 			} else
+ 				Conf_Server[i].lasttry = t;
 		}
 	}
 } /* Conf_UnsetServer */
@@ -256,8 +302,7 @@ Conf_GetServer( CONN_ID Idx )
 	
 	assert( Idx > NONE );
 
-	for( i = 0; i < MAX_SERVERS; i++ )
-	{
+	for( i = 0; i < MAX_SERVERS; i++ ) {
 		if( Conf_Server[i].conn_id == Idx ) return i;
 	}
 	return NONE;
@@ -273,10 +318,8 @@ Conf_EnableServer( char *Name, UINT16 Port )
 
 	assert( Name != NULL );
 
-	for( i = 0; i < MAX_SERVERS; i++ )
-	{
-		if( strcasecmp( Conf_Server[i].name, Name ) == 0 )
-		{
+	for( i = 0; i < MAX_SERVERS; i++ ) {
+		if( strcasecmp( Conf_Server[i].name, Name ) == 0 ) {
 			/* Gotcha! Set port and enable server: */
 			Conf_Server[i].port = Port;
 			Conf_Server[i].flags &= ~CONF_SFLAG_DISABLED;
@@ -296,10 +339,8 @@ Conf_DisableServer( char *Name )
 
 	assert( Name != NULL );
 
-	for( i = 0; i < MAX_SERVERS; i++ )
-	{
-		if( strcasecmp( Conf_Server[i].name, Name ) == 0 )
-		{
+	for( i = 0; i < MAX_SERVERS; i++ ) {
+		if( strcasecmp( Conf_Server[i].name, Name ) == 0 ) {
 			/* Gotcha! Disable and disconnect server: */
 			Conf_Server[i].flags |= CONF_SFLAG_DISABLED;
 			if( Conf_Server[i].conn_id > NONE ) Conn_Close( Conf_Server[i].conn_id, NULL, "Server link terminated on operator request", true);
@@ -323,8 +364,7 @@ Conf_AddServer( char *Name, UINT16 Port, char *Host, char *MyPwd, char *PeerPwd
 	assert( PeerPwd != NULL );
 
 	/* Search unused item in server configuration structure */
-	for( i = 0; i < MAX_SERVERS; i++ )
-	{
+	for( i = 0; i < MAX_SERVERS; i++ ) {
 		/* Is this item used? */
 		if( ! Conf_Server[i].name[0] ) break;
 	}
@@ -342,7 +382,7 @@ Conf_AddServer( char *Name, UINT16 Port, char *Host, char *MyPwd, char *PeerPwd
 } /* Conf_AddServer */
 
 
-LOCAL void
+static void
 Set_Defaults( bool InitServers )
 {
 	/* Initialize configuration variables with default values. */
@@ -366,7 +406,6 @@ Set_Defaults( bool InitServers )
 
 	strlcpy( Conf_PidFile, PID_FILE, sizeof( Conf_PidFile ));
 
-	Conf_ListenPorts_Count = 0;
 	strcpy( Conf_ListenAddress, "" );
 
 	Conf_UID = Conf_GID = 0;
@@ -391,21 +430,22 @@ Set_Defaults( bool InitServers )
 } /* Set_Defaults */
 
 
-LOCAL void
+static void
 Read_Config( void )
 {
 	/* Read configuration file. */
 
 	char section[LINE_LEN], str[LINE_LEN], *var, *arg, *ptr;
+	const UINT16 defaultport = 6667;
 	int line, i, n;
 	FILE *fd;
 
 	/* Open configuration file */
 	fd = fopen( NGIRCd_ConfFile, "r" );
-	if( ! fd )
-	{
+	if( ! fd ) {
 		/* No configuration file found! */
-		Config_Error( LOG_ALERT, "Can't read configuration \"%s\": %s", NGIRCd_ConfFile, strerror( errno ));
+		Config_Error( LOG_ALERT, "Can't read configuration \"%s\": %s",
+					NGIRCd_ConfFile, strerror( errno ));
 		Config_Error( LOG_ALERT, "%s exiting due to fatal errors!", PACKAGE_NAME );
 		exit( 1 );
 	}
@@ -418,28 +458,24 @@ Read_Config( void )
 	 * And delete all servers which are "duplicates" of servers
 	 * that are already marked as "once" (such servers have been
 	 * created by the last rehash but are now useless). */
-	for( i = 0; i < MAX_SERVERS; i++ )
-	{
+	for( i = 0; i < MAX_SERVERS; i++ ) {
 		if( Conf_Server[i].conn_id == NONE ) Init_Server_Struct( &Conf_Server[i] );
-		else
-		{
+		else {
 			/* This structure is in use ... */
-			if( Conf_Server[i].flags & CONF_SFLAG_ONCE )
-			{
+			if( Conf_Server[i].flags & CONF_SFLAG_ONCE ) {
 				/* Check for duplicates */
-				for( n = 0; n < MAX_SERVERS; n++ )
-				{
+				for( n = 0; n < MAX_SERVERS; n++ ) {
 					if( n == i ) continue;
 
-					if( Conf_Server[i].conn_id == Conf_Server[n].conn_id )
-					{
+					if( Conf_Server[i].conn_id == Conf_Server[n].conn_id ) {
 						Init_Server_Struct( &Conf_Server[n] );
-						Log( LOG_DEBUG, "Deleted unused duplicate server %d (kept %d).", n, i );
+#ifdef DEBUG
+						Log(LOG_DEBUG,"Deleted unused duplicate server %d (kept %d).",
+												n, i );
+#endif
 					}
 				}
-			}
-			else
-			{
+			} else {
 				/* Mark server as "once" */
 				Conf_Server[i].flags |= CONF_SFLAG_ONCE;
 				Log( LOG_DEBUG, "Marked server %d as \"once\"", i );
@@ -454,8 +490,7 @@ Read_Config( void )
 	New_Server_Idx = NONE;
 
 	/* Read configuration file */
-	while( true )
-	{
+	while( true ) {
 		if( ! fgets( str, LINE_LEN, fd )) break;
 		ngt_TrimStr( str );
 		line++;
@@ -464,15 +499,15 @@ Read_Config( void )
 		if( str[0] == ';' || str[0] == '#' || str[0] == '\0' ) continue;
 
 		/* Is this the beginning of a new section? */
-		if(( str[0] == '[' ) && ( str[strlen( str ) - 1] == ']' ))
-		{
+		if(( str[0] == '[' ) && ( str[strlen( str ) - 1] == ']' )) {
 			strlcpy( section, str, sizeof( section ));
-			if( strcasecmp( section, "[GLOBAL]" ) == 0 ) continue;
-			if( strcasecmp( section, "[OPERATOR]" ) == 0 )
-			{
-				if( Conf_Oper_Count + 1 > MAX_OPERATORS ) Config_Error( LOG_ERR, "Too many operators configured." );
-				else
-				{
+			if( strcasecmp( section, "[GLOBAL]" ) == 0 )
+				continue;
+
+			if( strcasecmp( section, "[OPERATOR]" ) == 0 ) {
+				if( Conf_Oper_Count + 1 > MAX_OPERATORS )
+					Config_Error( LOG_ERR, "Too many operators configured.");
+				else {
 					/* Initialize new operator structure */
 					Conf_Oper[Conf_Oper_Count].name[0] = '\0';
 					Conf_Oper[Conf_Oper_Count].pwd[0] = '\0';
@@ -484,11 +519,9 @@ Read_Config( void )
 				}
 				continue;
 			}
-			if( strcasecmp( section, "[SERVER]" ) == 0 )
-			{
+			if( strcasecmp( section, "[SERVER]" ) == 0 ) {
 				/* Check if there is already a server to add */
-				if( New_Server.name[0] )
-				{
+				if( New_Server.name[0] ) {
 					/* Copy data to "real" server structure */
 					assert( New_Server_Idx > NONE );
 					Conf_Server[New_Server_Idx] = New_Server;
@@ -498,13 +531,11 @@ Read_Config( void )
 				Init_Server_Struct( &New_Server );
 
 				/* Search unused item in server configuration structure */
-				for( i = 0; i < MAX_SERVERS; i++ )
-				{
+				for( i = 0; i < MAX_SERVERS; i++ ) {
 					/* Is this item used? */
 					if( ! Conf_Server[i].name[0] ) break;
 				}
-				if( i >= MAX_SERVERS )
-				{
+				if( i >= MAX_SERVERS ) {
 					/* Oops, no free item found! */
 					Config_Error( LOG_ERR, "Too many servers configured." );
 					New_Server_Idx = NONE;
@@ -512,15 +543,14 @@ Read_Config( void )
 				else New_Server_Idx = i;
 				continue;
 			}
-			if( strcasecmp( section, "[CHANNEL]" ) == 0 )
-			{
-				if( Conf_Channel_Count + 1 > MAX_DEFCHANNELS ) Config_Error( LOG_ERR, "Too many pre-defined channels configured." );
-				else
-				{
+			if( strcasecmp( section, "[CHANNEL]" ) == 0 ) {
+				if( Conf_Channel_Count + 1 > MAX_DEFCHANNELS ) {
+					Config_Error( LOG_ERR, "Too many pre-defined channels configured." );
+				} else {
 					/* Initialize new channel structure */
 					strcpy( Conf_Channel[Conf_Channel_Count].name, "" );
 					strcpy( Conf_Channel[Conf_Channel_Count].modes, "" );
-					strcpy( Conf_Channel[Conf_Channel_Count].topic, "" );
+					array_free(&Conf_Channel[Conf_Channel_Count].topic);
 					Conf_Channel_Count++;
 				}
 				continue;
@@ -532,8 +562,7 @@ Read_Config( void )
 
 		/* Split line into variable name and parameters */
 		ptr = strchr( str, '=' );
-		if( ! ptr )
-		{
+		if( ! ptr ) {
 			Config_Error( LOG_ERR, "%s, line %d: Syntax error!", NGIRCd_ConfFile, line );
 			continue;
 		}
@@ -552,23 +581,23 @@ Read_Config( void )
 	fclose( fd );
 
 	/* Check if there is still a server to add */
-	if( New_Server.name[0] )
-	{
+	if( New_Server.name[0] ) {
 		/* Copy data to "real" server structure */
 		assert( New_Server_Idx > NONE );
 		Conf_Server[New_Server_Idx] = New_Server;
 	}
-	
-	/* If there are no ports configured use the default: 6667 */
-	if( Conf_ListenPorts_Count < 1 )
-	{
-		Conf_ListenPorts_Count = 1;
-		Conf_ListenPorts[0] = 6667;
+
+	if (0 == array_length(&Conf_ListenPorts, sizeof(UINT16))) {
+		if (!array_copyb(&Conf_ListenPorts, (char*) &defaultport, sizeof defaultport)) {
+			Config_Error( LOG_ALERT, "Could not add default listening Port %u: %s",
+							(unsigned int) defaultport, strerror(errno));
+			exit( 1 );
+		}
 	}
 } /* Read_Config */
 
 
-LOCAL bool
+static bool
 Check_ArgIsTrue( const char *Arg )
 {
 	if( strcasecmp( Arg, "yes" ) == 0 ) return true;
@@ -579,120 +608,97 @@ Check_ArgIsTrue( const char *Arg )
 } /* Check_ArgIsTrue */
 
 
-LOCAL void
+static void
 Handle_GLOBAL( int Line, char *Var, char *Arg )
 {
 	struct passwd *pwd;
 	struct group *grp;
-	char *ptr;
-	long port;
+	size_t len;
 	
 	assert( Line > 0 );
 	assert( Var != NULL );
 	assert( Arg != NULL );
 	
-	if( strcasecmp( Var, "Name" ) == 0 )
-	{
+	if( strcasecmp( Var, "Name" ) == 0 ) {
 		/* Server name */
-		if( strlcpy( Conf_ServerName, Arg, sizeof( Conf_ServerName )) >= sizeof( Conf_ServerName ))
+		len = strlcpy( Conf_ServerName, Arg, sizeof( Conf_ServerName ));
+		if (len >= sizeof( Conf_ServerName ))
 			Config_Error_TooLong( Line, Var );
-
 		return;
 	}
-	if( strcasecmp( Var, "Info" ) == 0 )
-	{
+	if( strcasecmp( Var, "Info" ) == 0 ) {
 		/* Info text of server */
-		if( strlcpy( Conf_ServerInfo, Arg, sizeof( Conf_ServerInfo )) >= sizeof( Conf_ServerInfo ))
+		len = strlcpy( Conf_ServerInfo, Arg, sizeof( Conf_ServerInfo ));
+		if (len >= sizeof( Conf_ServerInfo ))
 			Config_Error_TooLong ( Line, Var );
-
 		return;
 	}
-	if( strcasecmp( Var, "Password" ) == 0 )
-	{
+	if( strcasecmp( Var, "Password" ) == 0 ) {
 		/* Global server password */
-		if( strlcpy( Conf_ServerPwd, Arg, sizeof( Conf_ServerPwd )) >= sizeof( Conf_ServerPwd ))
+		len = strlcpy( Conf_ServerPwd, Arg, sizeof( Conf_ServerPwd ));
+		if (len >= sizeof( Conf_ServerPwd ))
 			Config_Error_TooLong( Line, Var );
-
 		return;
 	}
-	if( strcasecmp( Var, "AdminInfo1" ) == 0 )
-	{
+	if( strcasecmp( Var, "AdminInfo1" ) == 0 ) {
 		/* Administrative info #1 */
-		if( strlcpy( Conf_ServerAdmin1, Arg, sizeof( Conf_ServerAdmin1 )) >= sizeof( Conf_ServerAdmin1 )) Config_Error_TooLong ( Line, Var );
+		len = strlcpy( Conf_ServerAdmin1, Arg, sizeof( Conf_ServerAdmin1 ));
+		if (len >= sizeof( Conf_ServerAdmin1 ))
+			Config_Error_TooLong ( Line, Var );
 		return;
 	}
-	if( strcasecmp( Var, "AdminInfo2" ) == 0 )
-	{
+	if( strcasecmp( Var, "AdminInfo2" ) == 0 ) {
 		/* Administrative info #2 */
-		if( strlcpy( Conf_ServerAdmin2, Arg, sizeof( Conf_ServerAdmin2 )) >= sizeof( Conf_ServerAdmin2 )) Config_Error_TooLong ( Line, Var );
+		len = strlcpy( Conf_ServerAdmin2, Arg, sizeof( Conf_ServerAdmin2 ));
+		if (len >= sizeof( Conf_ServerAdmin2 ))
+			Config_Error_TooLong ( Line, Var );
 		return;
 	}
-	if( strcasecmp( Var, "AdminEMail" ) == 0 )
-	{
+	if( strcasecmp( Var, "AdminEMail" ) == 0 ) {
 		/* Administrative email contact */
-		if( strlcpy( Conf_ServerAdminMail, Arg, sizeof( Conf_ServerAdminMail )) >= sizeof( Conf_ServerAdminMail )) Config_Error_TooLong( Line, Var );
+		len = strlcpy( Conf_ServerAdminMail, Arg, sizeof( Conf_ServerAdminMail ));
+		if (len >= sizeof( Conf_ServerAdminMail ))
+			Config_Error_TooLong( Line, Var );
 		return;
 	}
-	if( strcasecmp( Var, "Ports" ) == 0 )
-	{
-		/* Ports on that the server should listen. More port numbers
-		 * must be separated by "," */
-		ptr = strtok( Arg, "," );
-		while( ptr )
-		{
-			ngt_TrimStr( ptr );
-			port = atol( ptr );
-			if( Conf_ListenPorts_Count + 1 > MAX_LISTEN_PORTS ) Config_Error( LOG_ERR, "Too many listen ports configured. Port %ld ignored.", port );
-			else
-			{
-				if( port > 0 && port < 0xFFFF ) Conf_ListenPorts[Conf_ListenPorts_Count++] = (UINT16)port;
-				else Config_Error( LOG_ERR, "%s, line %d (section \"Global\"): Illegal port number %ld!", NGIRCd_ConfFile, Line, port );
-			}
-			ptr = strtok( NULL, "," );
-		}
+
+	if( strcasecmp( Var, "Ports" ) == 0 ) {
+		ports_parse(&Conf_ListenPorts, Line, Arg);
 		return;
 	}
-	if( strcasecmp( Var, "MotdFile" ) == 0 )
-	{
+	if( strcasecmp( Var, "MotdFile" ) == 0 ) {
 		/* "Message of the day" (MOTD) file */
-		if( strlcpy( Conf_MotdFile, Arg, sizeof( Conf_MotdFile )) >= sizeof( Conf_MotdFile ))
+		len = strlcpy( Conf_MotdFile, Arg, sizeof( Conf_MotdFile ));
+		if (len >= sizeof( Conf_MotdFile ))
 			Config_Error_TooLong( Line, Var );
-
 		return;
 	}
-	if( strcasecmp( Var, "MotdPhrase" ) == 0 )
-	{
+	if( strcasecmp( Var, "MotdPhrase" ) == 0 ) {
 		/* "Message of the day" phrase (instead of file) */
-		if( strlcpy( Conf_MotdPhrase, Arg, sizeof( Conf_MotdPhrase )) >= sizeof( Conf_MotdPhrase ))
+		len = strlcpy( Conf_MotdPhrase, Arg, sizeof( Conf_MotdPhrase ));
+		if (len >= sizeof( Conf_MotdPhrase ))
 			Config_Error_TooLong( Line, Var );
-
 		return;
 	}
-	if( strcasecmp( Var, "ChrootDir" ) == 0 )
-	{
+	if( strcasecmp( Var, "ChrootDir" ) == 0 ) {
 		/* directory for chroot() */
-		if( strlcpy( Conf_Chroot, Arg, sizeof( Conf_Chroot )) >= sizeof( Conf_Chroot ))
+		len = strlcpy( Conf_Chroot, Arg, sizeof( Conf_Chroot ));
+		if (len >= sizeof( Conf_Chroot ))
 			Config_Error_TooLong( Line, Var );
-
 		return;
 	}
-
-	if ( strcasecmp( Var, "PidFile" ) == 0 )
-	{
+	if ( strcasecmp( Var, "PidFile" ) == 0 ) {
 		/* name of pidfile */
-		if( strlcpy( Conf_PidFile, Arg, sizeof( Conf_PidFile )) >= sizeof( Conf_PidFile ))
+		len = strlcpy( Conf_PidFile, Arg, sizeof( Conf_PidFile ));
+		if (len >= sizeof( Conf_PidFile ))
 			Config_Error_TooLong( Line, Var );
-
 		return;
 	}
-
-	if( strcasecmp( Var, "ServerUID" ) == 0 )
-	{
+	if( strcasecmp( Var, "ServerUID" ) == 0 ) {
 		/* UID the daemon should switch to */
 		pwd = getpwnam( Arg );
 		if( pwd ) Conf_UID = pwd->pw_uid;
-		else
-		{
+		else {
 #ifdef HAVE_ISDIGIT
 			if( ! isdigit( (int)*Arg )) Config_Error_NaN( Line, Var );
 			else
@@ -701,13 +707,11 @@ Handle_GLOBAL( int Line, char *Var, char *Arg )
 		}
 		return;
 	}
-	if( strcasecmp( Var, "ServerGID" ) == 0 )
-	{
+	if( strcasecmp( Var, "ServerGID" ) == 0 ) {
 		/* GID the daemon should use */
 		grp = getgrnam( Arg );
 		if( grp ) Conf_GID = grp->gr_gid;
-		else
-		{
+		else {
 #ifdef HAVE_ISDIGIT
 			if( ! isdigit( (int)*Arg )) Config_Error_NaN( Line, Var );
 			else
@@ -716,53 +720,47 @@ Handle_GLOBAL( int Line, char *Var, char *Arg )
 		}
 		return;
 	}
-	if( strcasecmp( Var, "PingTimeout" ) == 0 )
-	{
+	if( strcasecmp( Var, "PingTimeout" ) == 0 ) {
 		/* PING timeout */
 		Conf_PingTimeout = atoi( Arg );
-		if( Conf_PingTimeout < 5 )
-		{
-			Config_Error( LOG_WARNING, "%s, line %d: Value of \"PingTimeout\" too low!", NGIRCd_ConfFile, Line );
+		if( Conf_PingTimeout < 5 ) {
+			Config_Error( LOG_WARNING, "%s, line %d: Value of \"PingTimeout\" too low!",
+									NGIRCd_ConfFile, Line );
 			Conf_PingTimeout = 5;
 		}
 		return;
 	}
-	if( strcasecmp( Var, "PongTimeout" ) == 0 )
-	{
+	if( strcasecmp( Var, "PongTimeout" ) == 0 ) {
 		/* PONG timeout */
 		Conf_PongTimeout = atoi( Arg );
-		if( Conf_PongTimeout < 5 )
-		{
-			Config_Error( LOG_WARNING, "%s, line %d: Value of \"PongTimeout\" too low!", NGIRCd_ConfFile, Line );
+		if( Conf_PongTimeout < 5 ) {
+			Config_Error( LOG_WARNING, "%s, line %d: Value of \"PongTimeout\" too low!",
+									NGIRCd_ConfFile, Line );
 			Conf_PongTimeout = 5;
 		}
 		return;
 	}
-	if( strcasecmp( Var, "ConnectRetry" ) == 0 )
-	{
+	if( strcasecmp( Var, "ConnectRetry" ) == 0 ) {
 		/* Seconds between connection attempts to other servers */
 		Conf_ConnectRetry = atoi( Arg );
-		if( Conf_ConnectRetry < 5 )
-		{
-			Config_Error( LOG_WARNING, "%s, line %d: Value of \"ConnectRetry\" too low!", NGIRCd_ConfFile, Line );
+		if( Conf_ConnectRetry < 5 ) {
+			Config_Error( LOG_WARNING, "%s, line %d: Value of \"ConnectRetry\" too low!",
+									NGIRCd_ConfFile, Line );
 			Conf_ConnectRetry = 5;
 		}
 		return;
 	}
-	if( strcasecmp( Var, "OperCanUseMode" ) == 0 )
-	{
+	if( strcasecmp( Var, "OperCanUseMode" ) == 0 ) {
 		/* Are IRC operators allowed to use MODE in channels they aren't Op in? */
 		Conf_OperCanMode = Check_ArgIsTrue( Arg );
 		return;
 	}
-	if( strcasecmp( Var, "OperServerMode" ) == 0 )
-	{
+	if( strcasecmp( Var, "OperServerMode" ) == 0 ) {
 		/* Mask IRC operator as if coming from the server? (ircd-irc2 compat hack) */
 		Conf_OperServerMode = Check_ArgIsTrue( Arg );
 		return;
 	}
-	if( strcasecmp( Var, "MaxConnections" ) == 0 )
-	{
+	if( strcasecmp( Var, "MaxConnections" ) == 0 ) {
 		/* Maximum number of connections. Values <= 0 are equal to "no limit". */
 #ifdef HAVE_ISDIGIT
 		if( ! isdigit( (int)*Arg )) Config_Error_NaN( Line, Var);
@@ -771,9 +769,8 @@ Handle_GLOBAL( int Line, char *Var, char *Arg )
 		Conf_MaxConnections = atol( Arg );
 		return;
 	}
-	if( strcasecmp( Var, "MaxConnectionsIP" ) == 0 )
-	{
-		/* Maximum number of simoultanous connections from one IP. Values <= 0 are equal to "no limit". */
+	if( strcasecmp( Var, "MaxConnectionsIP" ) == 0 ) {
+		/* Maximum number of simultaneous connections from one IP. Values <= 0 -> "no limit" */
 #ifdef HAVE_ISDIGIT
 		if( ! isdigit( (int)*Arg )) Config_Error_NaN( Line, Var );
 		else
@@ -781,8 +778,7 @@ Handle_GLOBAL( int Line, char *Var, char *Arg )
 		Conf_MaxConnectionsIP = atoi( Arg );
 		return;
 	}
-	if( strcasecmp( Var, "MaxJoins" ) == 0 )
-	{
+	if( strcasecmp( Var, "MaxJoins" ) == 0 ) {
 		/* Maximum number of channels a user can join. Values <= 0 are equal to "no limit". */
 #ifdef HAVE_ISDIGIT
 		if( ! isdigit( (int)*Arg )) Config_Error_NaN( Line, Var );
@@ -791,59 +787,64 @@ Handle_GLOBAL( int Line, char *Var, char *Arg )
 		Conf_MaxJoins = atoi( Arg );
 		return;
 	}
-	if( strcasecmp( Var, "Listen" ) == 0 )
-	{
+	if( strcasecmp( Var, "Listen" ) == 0 ) {
 		/* IP-Address to bind sockets */
-		if( strlcpy( Conf_ListenAddress, Arg, sizeof( Conf_ListenAddress )) >= sizeof( Conf_ListenAddress ))
-		{
+		len = strlcpy( Conf_ListenAddress, Arg, sizeof( Conf_ListenAddress ));
+		if (len >= sizeof( Conf_ListenAddress ))
 			Config_Error_TooLong( Line, Var );
-		}
 		return;
 	}
 
-	Config_Error( LOG_ERR, "%s, line %d (section \"Global\"): Unknown variable \"%s\"!", NGIRCd_ConfFile, Line, Var );
+	Config_Error( LOG_ERR, "%s, line %d (section \"Global\"): Unknown variable \"%s\"!",
+								NGIRCd_ConfFile, Line, Var );
 } /* Handle_GLOBAL */
 
 
-LOCAL void
+static void
 Handle_OPERATOR( int Line, char *Var, char *Arg )
 {
+	unsigned int opercount;
+	size_t len;
 	assert( Line > 0 );
 	assert( Var != NULL );
 	assert( Arg != NULL );
 	assert( Conf_Oper_Count > 0 );
 
-	if( strcasecmp( Var, "Name" ) == 0 )
-	{
-		/* Name of IRC operator */
-		if( strlcpy( Conf_Oper[Conf_Oper_Count - 1].name, Arg, sizeof( Conf_Oper[Conf_Oper_Count - 1].name )) >= sizeof( Conf_Oper[Conf_Oper_Count - 1].name )) Config_Error_TooLong( Line, Var );
+	if ( Conf_Oper_Count == 0 )
 		return;
-	}
-	if( strcasecmp( Var, "Password" ) == 0 )
-	{
-		/* Password of IRC operator */
-		if( strlcpy( Conf_Oper[Conf_Oper_Count - 1].pwd, Arg, sizeof( Conf_Oper[Conf_Oper_Count - 1].pwd )) >= sizeof( Conf_Oper[Conf_Oper_Count - 1].pwd )) Config_Error_TooLong( Line, Var );
-		return;
-	}
-	if( strcasecmp( Var, "Mask" ) == 0 )
-	{
-		if (Conf_Oper[Conf_Oper_Count - 1].mask) return; /* Hostname already configured */
-		Conf_Oper[Conf_Oper_Count - 1].mask = strdup( Arg );
-		if (! Conf_Oper[Conf_Oper_Count - 1].mask) {
-			Config_Error( LOG_ERR, "%s, line %d: Cannot allocate memory for operator mask: %s", NGIRCd_ConfFile, Line, strerror(errno) );
-			return;
-		}
 
+	opercount = Conf_Oper_Count - 1;
+
+	if( strcasecmp( Var, "Name" ) == 0 ) {
+		/* Name of IRC operator */
+		len = strlcpy( Conf_Oper[opercount].name, Arg, sizeof( Conf_Oper[opercount].name ));
+		if (len >= sizeof( Conf_Oper[opercount].name ))
+				Config_Error_TooLong( Line, Var );
 		return;
 	}
-	Config_Error( LOG_ERR, "%s, line %d (section \"Operator\"): Unknown variable \"%s\"!", NGIRCd_ConfFile, Line, Var );
+	if( strcasecmp( Var, "Password" ) == 0 ) {
+		/* Password of IRC operator */
+		len = strlcpy( Conf_Oper[opercount].pwd, Arg, sizeof( Conf_Oper[opercount].pwd ));
+		if (len >= sizeof( Conf_Oper[opercount].pwd ))
+				Config_Error_TooLong( Line, Var );
+		return;
+	}
+	if( strcasecmp( Var, "Mask" ) == 0 ) {
+		if (Conf_Oper[opercount].mask) return; /* Hostname already configured */
+
+		Conf_Oper[opercount].mask = strdup_warn( Arg );
+		return;
+	}
+	Config_Error( LOG_ERR, "%s, line %d (section \"Operator\"): Unknown variable \"%s\"!",
+								NGIRCd_ConfFile, Line, Var );
 } /* Handle_OPERATOR */
 
 
-LOCAL void
+static void
 Handle_SERVER( int Line, char *Var, char *Arg )
 {
 	long port;
+	size_t len;
 	
 	assert( Line > 0 );
 	assert( Var != NULL );
@@ -852,180 +853,210 @@ Handle_SERVER( int Line, char *Var, char *Arg )
 	/* Ignore server block if no space is left in server configuration structure */
 	if( New_Server_Idx <= NONE ) return;
 
-	if( strcasecmp( Var, "Host" ) == 0 )
-	{
+	if( strcasecmp( Var, "Host" ) == 0 ) {
 		/* Hostname of the server */
-		if( strlcpy( New_Server.host, Arg, sizeof( New_Server.host )) >= sizeof( New_Server.host ))
+		len = strlcpy( New_Server.host, Arg, sizeof( New_Server.host ));
+		if (len >= sizeof( New_Server.host ))
 			Config_Error_TooLong ( Line, Var );
-
 		return;
 	}
-	if( strcasecmp( Var, "Name" ) == 0 )
-	{
+	if( strcasecmp( Var, "Name" ) == 0 ) {
 		/* Name of the server ("Nick"/"ID") */
-		if( strlcpy( New_Server.name, Arg, sizeof( New_Server.name )) >= sizeof( New_Server.name ))
+		len = strlcpy( New_Server.name, Arg, sizeof( New_Server.name ));
+		if (len >= sizeof( New_Server.name ))
 			Config_Error_TooLong( Line, Var );
 		return;
 	}
-	if( strcasecmp( Var, "MyPassword" ) == 0 )
-	{
+	if( strcasecmp( Var, "MyPassword" ) == 0 ) {
 		/* Password of this server which is sent to the peer */
-		if( strlcpy( New_Server.pwd_in, Arg, sizeof( New_Server.pwd_in )) >= sizeof( New_Server.pwd_in )) Config_Error_TooLong( Line, Var );
+		if (*Arg == ':') {
+			Config_Error(LOG_ERR,
+				"%s, line %d (section \"Server\"): MyPassword must not start with ':'!",
+										NGIRCd_ConfFile, Line);
+		}
+		len = strlcpy( New_Server.pwd_in, Arg, sizeof( New_Server.pwd_in ));
+		if (len >= sizeof( New_Server.pwd_in ))
+			Config_Error_TooLong( Line, Var );
 		return;
 	}
-	if( strcasecmp( Var, "PeerPassword" ) == 0 )
-	{
+	if( strcasecmp( Var, "PeerPassword" ) == 0 ) {
 		/* Passwort of the peer which must be received */
-		if( strlcpy( New_Server.pwd_out, Arg, sizeof( New_Server.pwd_out )) >= sizeof( New_Server.pwd_out )) Config_Error_TooLong( Line, Var );
+		len = strlcpy( New_Server.pwd_out, Arg, sizeof( New_Server.pwd_out ));
+		if (len >= sizeof( New_Server.pwd_out ))
+			Config_Error_TooLong( Line, Var );
 		return;
 	}
-	if( strcasecmp( Var, "Port" ) == 0 )
-	{
+	if( strcasecmp( Var, "Port" ) == 0 ) {
 		/* Port to which this server should connect */
 		port = atol( Arg );
-		if( port > 0 && port < 0xFFFF ) New_Server.port = (UINT16)port;
-		else Config_Error( LOG_ERR, "%s, line %d (section \"Server\"): Illegal port number %ld!", NGIRCd_ConfFile, Line, port );
+		if( port > 0 && port < 0xFFFF )
+			New_Server.port = (UINT16)port;
+		else
+			Config_Error( LOG_ERR, "%s, line %d (section \"Server\"): Illegal port number %ld!",
+										NGIRCd_ConfFile, Line, port );
 		return;
 	}
-	if( strcasecmp( Var, "Group" ) == 0 )
-	{
+	if( strcasecmp( Var, "Group" ) == 0 ) {
 		/* Server group */
 #ifdef HAVE_ISDIGIT
-		if( ! isdigit( (int)*Arg )) Config_Error_NaN( Line, Var );
+		if( ! isdigit( (int)*Arg ))
+			Config_Error_NaN( Line, Var );
 		else
 #endif
 		New_Server.group = atoi( Arg );
 		return;
 	}
 	
-	Config_Error( LOG_ERR, "%s, line %d (section \"Server\"): Unknown variable \"%s\"!", NGIRCd_ConfFile, Line, Var );
+	Config_Error( LOG_ERR, "%s, line %d (section \"Server\"): Unknown variable \"%s\"!",
+								NGIRCd_ConfFile, Line, Var );
 } /* Handle_SERVER */
 
 
-LOCAL void
+static void
 Handle_CHANNEL( int Line, char *Var, char *Arg )
 {
+	size_t len;
+	size_t chancount = 0;
+
 	assert( Line > 0 );
 	assert( Var != NULL );
 	assert( Arg != NULL );
+	if (Conf_Channel_Count > 0)
+		chancount = Conf_Channel_Count - 1;
 
-	if( strcasecmp( Var, "Name" ) == 0 )
-	{
+	if( strcasecmp( Var, "Name" ) == 0 ) {
 		/* Name of the channel */
-		if( strlcpy( Conf_Channel[Conf_Channel_Count - 1].name, Arg, sizeof( Conf_Channel[Conf_Channel_Count - 1].name )) >= sizeof( Conf_Channel[Conf_Channel_Count - 1].name ))
+		len = strlcpy( Conf_Channel[chancount].name, Arg, sizeof( Conf_Channel[chancount].name ));
+		if (len >= sizeof( Conf_Channel[chancount].name ))
 			Config_Error_TooLong( Line, Var );
 		return;
 	}
-	if( strcasecmp( Var, "Modes" ) == 0 )
-	{
+	if( strcasecmp( Var, "Modes" ) == 0 ) {
 		/* Initial modes */
-		if( strlcpy( Conf_Channel[Conf_Channel_Count - 1].modes, Arg, sizeof( Conf_Channel[Conf_Channel_Count - 1].modes )) >= sizeof( Conf_Channel[Conf_Channel_Count - 1].modes ))
+		len = strlcpy( Conf_Channel[chancount].modes, Arg, sizeof( Conf_Channel[chancount].modes ));
+		if (len >= sizeof( Conf_Channel[chancount].modes ))
 			Config_Error_TooLong( Line, Var );
 		return;
 	}
-	if( strcasecmp( Var, "Topic" ) == 0 )
-	{
+	if( strcasecmp( Var, "Topic" ) == 0 ) {
 		/* Initial topic */
-		if( strlcpy( Conf_Channel[Conf_Channel_Count - 1].topic, Arg, sizeof( Conf_Channel[Conf_Channel_Count - 1].topic )) >= sizeof( Conf_Channel[Conf_Channel_Count - 1].topic ))
+		if (!array_copys( &Conf_Channel[chancount].topic, Arg))
 			Config_Error_TooLong( Line, Var );
- 
 		return;
 	}
 
-	Config_Error( LOG_ERR, "%s, line %d (section \"Channel\"): Unknown variable \"%s\"!", NGIRCd_ConfFile, Line, Var );
+	Config_Error( LOG_ERR, "%s, line %d (section \"Channel\"): Unknown variable \"%s\"!",
+								NGIRCd_ConfFile, Line, Var );
 } /* Handle_CHANNEL */
 
 
-LOCAL void
-Validate_Config( bool Configtest )
+static void
+Validate_Config(bool Configtest, bool Rehash)
 {
 	/* Validate configuration settings. */
 
 #ifdef DEBUG
 	int i, servers, servers_once;
 #endif
+	char *ptr;
 
-	if( ! Conf_ServerName[0] )
-	{
+	/* Validate configured server name, see RFC 2812 section 2.3.1 */
+	ptr = Conf_ServerName;
+	do {
+		if (*ptr >= 'a' && *ptr <= 'z') continue;
+		if (*ptr >= 'A' && *ptr <= 'Z') continue;
+		if (*ptr >= '1' && *ptr <= '0') continue;
+		if (ptr > Conf_ServerName) {
+			if (*ptr == '.' || *ptr == '-')
+				continue;
+		}
+		Conf_ServerName[0] = '\0';
+		break;
+	} while (*(++ptr));
+
+	if (!Conf_ServerName[0]) {
 		/* No server name configured! */
-		Config_Error( LOG_ALERT, "No server name configured in \"%s\" (section 'Global': 'Name')!", NGIRCd_ConfFile );
-		if( ! Configtest )
-		{
-			Config_Error( LOG_ALERT, "%s exiting due to fatal errors!", PACKAGE_NAME );
-			exit( 1 );
+		Config_Error(LOG_ALERT,
+			     "No (valid) server name configured in \"%s\" (section 'Global': 'Name')!",
+			     NGIRCd_ConfFile);
+		if (!Configtest && !Rehash) {
+			Config_Error(LOG_ALERT,
+				     "%s exiting due to fatal errors!",
+				     PACKAGE_NAME);
+			exit(1);
 		}
 	}
-	
-	if( Conf_ServerName[0] && ! strchr( Conf_ServerName, '.' ))
-	{
+
+	if (Conf_ServerName[0] && !strchr(Conf_ServerName, '.')) {
 		/* No dot in server name! */
-		Config_Error( LOG_ALERT, "Invalid server name configured in \"%s\" (section 'Global': 'Name'): Dot missing!", NGIRCd_ConfFile );
-		if( ! Configtest )
-		{
-			Config_Error( LOG_ALERT, "%s exiting due to fatal errors!", PACKAGE_NAME );
-			exit( 1 );
+		Config_Error(LOG_ALERT,
+			     "Invalid server name configured in \"%s\" (section 'Global': 'Name'): Dot missing!",
+			     NGIRCd_ConfFile);
+		if (!Configtest) {
+			Config_Error(LOG_ALERT,
+				     "%s exiting due to fatal errors!",
+				     PACKAGE_NAME);
+			exit(1);
 		}
 	}
 
 #ifdef STRICT_RFC
-	if( ! Conf_ServerAdminMail[0] )
-	{
+	if (!Conf_ServerAdminMail[0]) {
 		/* No administrative contact configured! */
-		Config_Error( LOG_ALERT, "No administrator email address configured in \"%s\" ('AdminEMail')!", NGIRCd_ConfFile );
-		if( ! Configtest )
-		{
-			Config_Error( LOG_ALERT, "%s exiting due to fatal errors!", PACKAGE_NAME );
-			exit( 1 );
+		Config_Error(LOG_ALERT,
+			     "No administrator email address configured in \"%s\" ('AdminEMail')!",
+			     NGIRCd_ConfFile);
+		if (!Configtest) {
+			Config_Error(LOG_ALERT,
+				     "%s exiting due to fatal errors!",
+				     PACKAGE_NAME);
+			exit(1);
 		}
 	}
 #endif
 
-	if( ! Conf_ServerAdmin1[0] && ! Conf_ServerAdmin2[0] && ! Conf_ServerAdminMail[0] )
-	{
+	if (!Conf_ServerAdmin1[0] && !Conf_ServerAdmin2[0]
+	    && !Conf_ServerAdminMail[0]) {
 		/* No administrative information configured! */
-		Config_Error( LOG_WARNING, "No administrative information configured but required by RFC!" );
+		Config_Error(LOG_WARNING,
+			     "No administrative information configured but required by RFC!");
 	}
-#ifdef FD_SETSIZE	
-	if(( Conf_MaxConnections > (long)FD_SETSIZE ) || ( Conf_MaxConnections < 1 ))
-	{
-		Conf_MaxConnections = (long)FD_SETSIZE;
-		Config_Error( LOG_ERR, "Setting MaxConnections to %ld, select() can't handle more file descriptors!", Conf_MaxConnections );
-	}
-#else
-	Config_Error( LOG_WARN, "Don't know how many file descriptors select() can handle on this system, don't set MaxConnections too high!" );
-#endif
 
 #ifdef DEBUG
 	servers = servers_once = 0;
-	for( i = 0; i < MAX_SERVERS; i++ )
-	{
-		if( Conf_Server[i].name[0] )
-		{
+	for (i = 0; i < MAX_SERVERS; i++) {
+		if (Conf_Server[i].name[0]) {
 			servers++;
-			if( Conf_Server[i].flags & CONF_SFLAG_ONCE ) servers_once++;
+			if (Conf_Server[i].flags & CONF_SFLAG_ONCE)
+				servers_once++;
 		}
 	}
-	Log( LOG_DEBUG, "Configuration: Operators=%d, Servers=%d[%d], Channels=%d", Conf_Oper_Count, servers, servers_once, Conf_Channel_Count );
+	Log(LOG_DEBUG,
+	    "Configuration: Operators=%d, Servers=%d[%d], Channels=%d",
+	    Conf_Oper_Count, servers, servers_once, Conf_Channel_Count);
 #endif
 } /* Validate_Config */
 
 
-LOCAL void
+static void
 Config_Error_TooLong ( const int Line, const char *Item )
 {
 	Config_Error( LOG_WARNING, "%s, line %d: Value of \"%s\" too long!", NGIRCd_ConfFile, Line, Item );
 }
 
-LOCAL void
+
+static void
 Config_Error_NaN( const int Line, const char *Item )
 {
-	Config_Error( LOG_WARNING, "%s, line %d: Value of \"%s\" is not a number!", NGIRCd_ConfFile, Line, Item );
+	Config_Error( LOG_WARNING, "%s, line %d: Value of \"%s\" is not a number!",
+						NGIRCd_ConfFile, Line, Item );
 }
 
+
 #ifdef PROTOTYPES
-LOCAL void Config_Error( const int Level, const char *Format, ... )
+static void Config_Error( const int Level, const char *Format, ... )
 #else
-LOCAL void Config_Error( Level, Format, va_alist )
+static void Config_Error( Level, Format, va_alist )
 const int Level;
 const char *Format;
 va_dcl
@@ -1049,12 +1080,12 @@ va_dcl
 	/* During "normal operations" the log functions of the daemon should
 	 * be used, but during testing of the configuration file, all messages
 	 * should go directly to the console: */
-	if( Use_Log ) Log( Level, "%s", msg );
+	if (Use_Log) Log( Level, "%s", msg );
 	else puts( msg );
 } /* Config_Error */
 
 
-LOCAL void
+static void
 Init_Server_Struct( CONF_SERVER *Server )
 {
 	/* Initialize server configuration structur to default values */
@@ -1068,6 +1099,7 @@ Init_Server_Struct( CONF_SERVER *Server )
 
 	if( NGIRCd_Passive ) Server->flags = CONF_SFLAG_DISABLED;
 
+	Resolve_Init(&Server->res_stat);
 	Server->conn_id = NONE;
 } /* Init_Server_Struct */
 
