@@ -14,7 +14,7 @@
 
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: log.c,v 1.57.2.1 2005/08/29 11:19:48 alex Exp $";
+static char UNUSED id[] = "$Id: log.c,v 1.61 2006/07/23 23:23:45 alex Exp $";
 
 #include "imp.h"
 #include <assert.h>
@@ -44,15 +44,15 @@ static char UNUSED id[] = "$Id: log.c,v 1.57.2.1 2005/08/29 11:19:48 alex Exp $"
 #include "log.h"
 
 
-LOCAL char Init_Txt[127];
-LOCAL bool Is_Daemon;
+static char Init_Txt[127];
+static bool Is_Daemon;
 
 #ifdef DEBUG
-LOCAL char Error_File[FNAME_LEN];
+static char Error_File[FNAME_LEN];
 #endif
 
 
-LOCAL void Wall_ServerNotice PARAMS(( char *Msg ));
+static void Wall_ServerNotice PARAMS(( char *Msg ));
 
 
 GLOBAL void
@@ -162,6 +162,60 @@ Log_Exit( void )
 } /* Log_Exit */
 
 
+/**
+ * Log function for debug messages.
+ * This function is only functional when the program is compiled with debug
+ * code enabled; otherwise it is an empty function which the compiler will
+ * hopefully mangle down to "nothing". Therefore you should use LogDebug(...)
+ * in favor to Log(LOG_DEBUG, ...).
+ * @param Format Format string like printf().
+ * @param ... Further arguments.
+ */
+# ifdef PROTOTYPES
+GLOBAL void
+#ifdef DEBUG
+LogDebug( const char *Format, ... )
+#else
+LogDebug( UNUSED const char *Format, ... )
+#endif /* DEBUG */
+# else
+GLOBAL void
+LogDebug( Format, va_alist )
+const char *Format;
+va_dcl
+# endif /* PROTOTYPES */
+#ifdef DEBUG
+{
+	char msg[MAX_LOG_MSG_LEN];
+	va_list ap;
+
+	if (!NGIRCd_Debug) return;
+#ifdef PROTOTYPES
+	va_start( ap, Format );
+#else
+	va_start( ap );
+#endif
+	vsnprintf( msg, MAX_LOG_MSG_LEN, Format, ap );
+	va_end( ap );
+	Log(LOG_DEBUG, "%s", msg);
+}
+#else
+{
+	/* Do nothing.
+	 * The compiler should optimize this out, please ;-) */
+}
+#endif	/* DEBUG */
+
+	
+/**
+ * Logging function of ngIRCd.
+ * This function logs messages to the console and/or syslog, whichever is
+ * suitable for the mode ngIRCd is running in (daemon vs. non-daemon).
+ * Please note: you sould use LogDebug(...) for debug messages!
+ * @param Level syslog level (LOG_xxx)
+ * @param Format Format string like printf().
+ * @param ... Further arguments.
+ */
 #ifdef PROTOTYPES
 GLOBAL void
 Log( int Level, const char *Format, ... )
@@ -174,7 +228,6 @@ va_dcl
 #endif
 {
 	/* Eintrag in Logfile(s) schreiben */
-
 	char msg[MAX_LOG_MSG_LEN];
 	bool snotice;
 	va_list ap;
@@ -306,7 +359,7 @@ va_dcl
  * Send log messages to users flagged with the "s" mode.
  * @param Msg The message to send.
  */
-LOCAL void
+static void
 Wall_ServerNotice( char *Msg )
 {
 	CLIENT *c;
