@@ -1,6 +1,6 @@
 /*
  * ngIRCd -- The Next Generation IRC Daemon
- * Copyright (c)2001-2005 by Alexander Barton (alex@barton.de)
+ * Copyright (c)2001-2006 Alexander Barton (alex@barton.de).
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: ngircd.c,v 1.102.2.2 2005/07/08 23:20:04 alex Exp $";
+static char UNUSED id[] = "$Id: ngircd.c,v 1.113 2006/07/23 12:07:33 alex Exp $";
 
 /**
  * @file
@@ -48,7 +48,7 @@ static char UNUSED id[] = "$Id: ngircd.c,v 1.102.2.2 2005/07/08 23:20:04 alex Ex
 #include "parse.h"
 #include "irc.h"
 
-#ifdef RENDEZVOUS
+#ifdef ZEROCONF
 #include "rendezvous.h"
 #endif
 
@@ -56,20 +56,20 @@ static char UNUSED id[] = "$Id: ngircd.c,v 1.102.2.2 2005/07/08 23:20:04 alex Ex
 #include "ngircd.h"
 
 
-LOCAL void Initialize_Signal_Handler PARAMS(( void ));
-LOCAL void Signal_Handler PARAMS(( int Signal ));
+static void Initialize_Signal_Handler PARAMS(( void ));
+static void Signal_Handler PARAMS(( int Signal ));
 
-LOCAL void Show_Version PARAMS(( void ));
-LOCAL void Show_Help PARAMS(( void ));
+static void Show_Version PARAMS(( void ));
+static void Show_Help PARAMS(( void ));
 
-LOCAL void Pidfile_Create PARAMS(( long ));
-LOCAL void Pidfile_Delete PARAMS(( void ));
+static void Pidfile_Create PARAMS(( pid_t pid ));
+static void Pidfile_Delete PARAMS(( void ));
 
-LOCAL void Fill_Version PARAMS(( void ));
+static void Fill_Version PARAMS(( void ));
 
-LOCAL void Setup_FDStreams PARAMS(( void ));
+static void Setup_FDStreams PARAMS(( void ));
 
-LOCAL bool NGIRCd_Init PARAMS(( bool ));
+static bool NGIRCd_Init PARAMS(( bool ));
 
 /**
  * The main() function of ngIRCd.
@@ -262,18 +262,19 @@ main( int argc, const char *argv[] )
 		Log_Init( ! NGIRCd_NoDaemon );
 		Conf_Init( );
 
-		if (!NGIRCd_Init( NGIRCd_NoDaemon )) {
-			Log(LOG_WARNING, "Fatal: Initialization failed");
+		/* Initialize the "main program": chroot environment, user and
+		 * group ID, ... */
+		if (!NGIRCd_Init(NGIRCd_NoDaemon)) {
+			Log(LOG_ALERT, "Fatal: Initialization failed");
 			exit(1);
 		}
 
 		/* Initialize modules, part II: these functions are eventually
 		 * called with already dropped privileges ... */
-		Resolve_Init( );
 		Lists_Init( );
 		Channel_Init( );
 		Client_Init( );
-#ifdef RENDEZVOUS
+#ifdef ZEROCONF
 		Rendezvous_Init( );
 #endif
 		Conn_Init( );
@@ -322,7 +323,7 @@ main( int argc, const char *argv[] )
 
 		/* Alles abmelden */
 		Conn_Exit( );
-#ifdef RENDEZVOUS
+#ifdef ZEROCONF
 		Rendezvous_Exit( );
 #endif
 		Client_Exit( );
@@ -342,7 +343,7 @@ main( int argc, const char *argv[] )
  * further usage, for example by the IRC command VERSION and the --version
  * command line switch.
  */
-LOCAL void
+static void
 Fill_Version( void )
 {
 	NGIRCd_VersionAddition[0] = '\0';
@@ -362,11 +363,11 @@ Fill_Version( void )
 
 	strlcat( NGIRCd_VersionAddition, "TCPWRAP", sizeof NGIRCd_VersionAddition );
 #endif
-#ifdef RENDEZVOUS
+#ifdef ZEROCONF
 	if( NGIRCd_VersionAddition[0] )
 		strlcat( NGIRCd_VersionAddition, "+", sizeof NGIRCd_VersionAddition );
 
-	strlcat( NGIRCd_VersionAddition, "RENDEZVOUS", sizeof NGIRCd_VersionAddition );
+	strlcat( NGIRCd_VersionAddition, "ZEROCONF", sizeof NGIRCd_VersionAddition );
 #endif
 #ifdef IDENTAUTH
 	if( NGIRCd_VersionAddition[0] )
@@ -439,7 +440,7 @@ NGIRCd_Rehash( void )
 	/* Recover old server name: it can't be changed during run-time */
 	if( strcmp( old_name, Conf_ServerName ) != 0 )
 	{
-		strcpy( Conf_ServerName, old_name );
+		strlcpy( Conf_ServerName, old_name, sizeof Conf_ServerName );
 		Log( LOG_ERR, "Can't change \"ServerName\" on runtime! Ignored new name." );
 	}
 
@@ -459,7 +460,7 @@ NGIRCd_Rehash( void )
 /**
  * Initialize the signal handler.
  */
-LOCAL void
+static void
 Initialize_Signal_Handler( void )
 {
 	/* Signal-Handler initialisieren: einige Signale
@@ -481,27 +482,27 @@ Initialize_Signal_Handler( void )
 #endif
 
 	/* Signal-Handler einhaengen */
-	sigaction( SIGINT, &saction, NULL );
-	sigaction( SIGQUIT, &saction, NULL );
-	sigaction( SIGTERM, &saction, NULL);
-	sigaction( SIGHUP, &saction, NULL);
-	sigaction( SIGCHLD, &saction, NULL);
+	sigaction(SIGINT, &saction, NULL);
+	sigaction(SIGQUIT, &saction, NULL);
+	sigaction(SIGTERM, &saction, NULL);
+	sigaction(SIGHUP, &saction, NULL);
+	sigaction(SIGCHLD, &saction, NULL);
 
 	/* einige Signale ignorieren */
 	saction.sa_handler = SIG_IGN;
-	sigaction( SIGPIPE, &saction, NULL );
+	sigaction(SIGPIPE, &saction, NULL);
 #else
 	/* kein sigaction() vorhanden */
 
 	/* Signal-Handler einhaengen */
-	signal( SIGINT, Signal_Handler );
-	signal( SIGQUIT, Signal_Handler );
-	signal( SIGTERM, Signal_Handler );
-	signal( SIGHUP, Signal_Handler );
-	signal( SIGCHLD, Signal_Handler );
+	signal(SIGINT, Signal_Handler);
+	signal(SIGQUIT, Signal_Handler);
+	signal(SIGTERM, Signal_Handler);
+	signal(SIGHUP, Signal_Handler);
+	signal(SIGCHLD, Signal_Handler);
 
 	/* einige Signale ignorieren */
-	signal( SIGPIPE, SIG_IGN );
+	signal(SIGPIPE, SIG_IGN);
 #endif
 } /* Initialize_Signal_Handler */
 
@@ -512,7 +513,7 @@ Initialize_Signal_Handler( void )
  * user and/or the system to it. For example SIGTERM and SIGHUP.
  * @param Signal Number of the signal to handle.
  */
-LOCAL void
+static void
 Signal_Handler( int Signal )
 {
 	switch( Signal )
@@ -543,12 +544,12 @@ Signal_Handler( int Signal )
 /**
  * Display copyright and version information of ngIRCd on the console.
  */
-LOCAL void
+static void
 Show_Version( void )
 {
 	puts( NGIRCd_Version );
-	puts( "Copyright (c)2001-2005 by Alexander Barton (<alex@barton.de>)." );
-	puts( "Homepage: <http://arthur.ath.cx/~alex/ngircd/>\n" );
+	puts( "Copyright (c)2001-2006 Alexander Barton (<alex@barton.de>) and Contributors." );
+	puts( "Homepage: <http://ngircd.barton.de/>\n" );
 	puts( "This is free software; see the source for copying conditions. There is NO" );
 	puts( "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE." );
 } /* Show_Version */
@@ -559,7 +560,7 @@ Show_Version( void )
  * This help depends on the configuration of the executable and only shows
  * options that are actually enabled.
  */
-LOCAL void
+static void
 Show_Help( void )
 {
 #ifdef DEBUG
@@ -580,7 +581,7 @@ Show_Help( void )
 /**
  * Delete the file containing the process ID (PID).
  */
-LOCAL void
+static void
 Pidfile_Delete( void )
 {
 	/* Pidfile configured? */
@@ -599,8 +600,8 @@ Pidfile_Delete( void )
  * Create the file containing the process ID of ngIRCd ("PID file").
  * @param pid The process ID to be stored in this file.
  */
-LOCAL void
-Pidfile_Create( long pid )
+static void
+Pidfile_Create(pid_t pid)
 {
 	int pidfd;
 	char pidbuf[64];
@@ -619,13 +620,13 @@ Pidfile_Create( long pid )
 		return;
 	}
 
-	len = snprintf( pidbuf, sizeof pidbuf, "%ld\n", pid );
-	if (len < 0|| len < (int)sizeof pid) {
+	len = snprintf(pidbuf, sizeof pidbuf, "%ld\n", (long)pid);
+	if (len < 0 || len >= (int)sizeof pidbuf) {
 		Log( LOG_ERR, "Error converting pid");
 		return;
 	}
 	
-	if( write( pidfd, pidbuf, len) != len)
+	if (write(pidfd, pidbuf, (size_t)len) != (ssize_t)len)
 		Log( LOG_ERR, "Can't write PID file (%s): %s", Conf_PidFile, strerror( errno ));
 
 	if( close(pidfd) != 0 )
@@ -636,7 +637,7 @@ Pidfile_Create( long pid )
 /**
  * Redirect stdin, stdout and stderr to apropriate file handles.
  */
-LOCAL void
+static void
 Setup_FDStreams( void )
 {
 	int fd;
@@ -661,8 +662,8 @@ Setup_FDStreams( void )
 } /* Setup_FDStreams */
 
 
-LOCAL bool
-NGIRCd_getNobodyID(unsigned int *uid, unsigned int *gid )
+static bool
+NGIRCd_getNobodyID(uid_t *uid, gid_t *gid )
 {
 	struct passwd *pwd;
 
@@ -680,7 +681,7 @@ NGIRCd_getNobodyID(unsigned int *uid, unsigned int *gid )
 }
 
 
-LOCAL bool
+static bool
 NGIRCd_Init( bool NGIRCd_NoDaemon ) 
 {
 	static bool initialized;
@@ -688,7 +689,7 @@ NGIRCd_Init( bool NGIRCd_NoDaemon )
 	struct passwd *pwd;
 	struct group *grp;
 	int real_errno;
-	long pid;
+	pid_t pid;
 
 	if (initialized)
 		return true;
@@ -748,7 +749,7 @@ NGIRCd_Init( bool NGIRCd_NoDaemon )
 	 * connected to ther controlling terminal. Use "--nodaemon"
 	 * to disable this "daemon mode" (useful for debugging). */
 	if ( ! NGIRCd_NoDaemon ) {
-		pid = (long)fork( );
+		pid = fork( );
 		if( pid > 0 ) {
 			/* "Old" process: exit. */
 			exit( 0 );
